unit uImageButton;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, GraphType, Graphics, Controls, Types, ImgList;

type
  TImageButton = class(TGraphicControl)
  private
    type
      TImageButtonStatus = set of (ibsMouseDown, ibsHot);
  private
    FStatus: TImageButtonStatus;
    FImages: TCustomImageList;
    FImagesWidth: Integer;
    FImageIndex: TImageIndex;
    procedure SetImageIndex(AValue: TImageIndex);
    procedure SetImages(AValue: TCustomImageList);
  protected
    class function GetControlClassDefaultSize: TSize; override;
    procedure MouseDown(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
    procedure MouseUp(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;

  public
    constructor Create(AOwner: TComponent); override;
  published
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
    property Images: TCustomImageList read FImages write SetImages;
    property ImagesWidth: Integer read FImagesWidth write FImagesWidth default 0;
    property Align;
    property Anchors;
    property BorderSpacing;
    property Height;
    property Width;
    property OnClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseUp;
  end;

function AddImgButton(AOwner: TComponent; AParent: TWinControl;
  X, Y: Integer; AImages: TCustomImageList; AImageIndex: Integer;
  AOnClick: TNotifyEvent): TImageButton;

procedure Register;

implementation

{$R imagebutton_img.res}

procedure Register;
begin
  RegisterComponents('Misc', [TImageButton]);
end;

function AddImgButton(AOwner: TComponent; AParent: TWinControl;
  X, Y: Integer; AImages: TCustomImageList; AImageIndex: Integer;
  AOnClick: TNotifyEvent): TImageButton;
begin
  Result := TImageButton.Create(AOwner);
  Result.Left := X;
  Result.Top := Y;
  Result.Parent := AParent;
  Result.Images := AImages;
  Result.ImageIndex := AImageIndex;
  Result.OnClick := AOnClick;
end;

constructor TImageButton.Create(AOwner: TComponent);
begin
  inherited;
  FImageIndex := -1;
  with GetControlClassDefaultSize do
    SetInitialBounds(0, 0, CX, CY);
end;

class function TImageButton.GetControlClassDefaultSize: TSize;
begin
  Result.CX := 24;
  Result.CY := 24;
end;

procedure TImageButton.MouseDown(Button: TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
  inherited;
  if Button = mbLeft then
  begin
    FStatus := FStatus + [ibsMouseDown];
    Invalidate;
  end;
end;

procedure TImageButton.MouseEnter;
begin
  FStatus := FStatus + [ibsHot];
  Invalidate;
  inherited;
end;

procedure TImageButton.MouseLeave;
begin
  FStatus := FStatus - [ibsHot];
  Invalidate;
  inherited;
end;

procedure TImageButton.MouseUp(Button: TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
  inherited;
  if Button = mbLeft then
  begin
    FStatus := FStatus - [ibsMouseDown];
    Invalidate;
  end;
end;

procedure TImageButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FImages) then
    FImages := nil;
end;

procedure TImageButton.Paint;
var
  effect: TGraphicsDrawEffect;
  x, y, w, h: Integer;
begin
  inherited;

  if (csDesigning in ComponentState) and ((FImages = nil) or (FImageIndex = -1)) then
  begin
    Canvas.Pen.Color := clBlack;
    Canvas.Rectangle(0, 0, Width, Height);
    Canvas.Line(0, 0, Width, Height);
    Canvas.Line(0, Height, Width, 0);
    exit;
  end;

  if Assigned(FImages) and (FImageIndex > -1) then
  begin
    if csDesigning in ComponentState then
      effect := gdeNormal
    else
    if ibsMouseDown in FStatus then
      effect := gdeShadowed
    else
    if ibsHot in FStatus then
      effect := gdeHighlighted
    else
      effect := gdeNormal;
    w := FImages.WidthForPPI[FImagesWidth, Font.PixelsPerInch];
    h := FImages.HeightForPPI[FImagesWidth, Font.PixelsPerInch];
    x := (Width - w) div 2;
    y := (Height - h) div 2;
    FImages.Draw(Canvas, x, y, FImageIndex, effect);
  end;
end;

procedure TImageButton.SetImageIndex(AValue: TImageIndex);
begin
  if AValue <> FImageIndex then
  begin
    FImageIndex := AValue;
    Invalidate;
  end;
end;

procedure TImageButton.SetImages(AValue: TCustomImageList);
begin
  if AValue <> FImages then
  begin
    FImages := AValue;
    Invalidate;
  end;
end;

end.

